@@ -5,13 +5,13 @@ import _thread
 import numpy as np
 import threading
 from rpyc import Service
-from time import sleep, time
+from time import sleep
 from random import random
 from rpyc.utils.server import OneShotServer
 from PyRedPitaya.board import RedPitaya
 
 sys.path += ['../../']
-from csr import make_filter, PitayaLocal
+from csr import PythonCSR
 from linien.config import ACQUISITION_PORT
 from linien.common import DECIMATION, N_POINTS
 
@@ -29,15 +29,16 @@ def decimate(array, factor):
     if factor == 1:
         return array
 
-    array = np.array(array)
-    array = list(array.reshape(-1, factor).mean(axis=1))
-    return array
+    dtype = array.dtype
+    return np.round(
+        array.reshape(-1, factor).mean(axis=1)
+    ).astype(dtype)
 
 
 class DataAcquisitionService(Service):
     def __init__(self):
         self.r = RedPitaya()
-        self.csr = PitayaLocal()
+        self.csr = PythonCSR(self.r)
         self.csr_queue = []
         self.csr_iir_queue = []
 
@@ -51,6 +52,14 @@ class DataAcquisitionService(Service):
 
         self.exposed_set_ramp_speed(9)
         self.locked = False
+        # when self.locked is set to True, this doesn't mean that the lock is
+        # really on. It just means that the lock is requested and that the
+        # gateware waits until the sweep is at the correct position for the lock.
+        # Therefore, when self.locked is set, the acquisition process waits for
+        # confirmation from the gateware that the lock is actually running.
+        self.confirmed_that_in_lock = False
+
+        self.fetch_quadratures = True
 
         self.run()
 
@@ -65,6 +74,12 @@ class DataAcquisitionService(Service):
                     args = self.csr_iir_queue.pop(0)
                     self.csr.set_iir(*args)
 
+                if self.locked and not self.confirmed_that_in_lock:
+                    self.confirmed_that_in_lock = self.csr.get('logic_lock_running')
+                    if not self.confirmed_that_in_lock:
+                        sleep(.05)
+                        continue
+
                 # copied from https://github.com/RedPitaya/RedPitaya/blob/14cca62dd58f29826ee89f4b28901602f5cdb1d8/api/src/oscilloscope.c#L115
                 # check whether scope was triggered
                 if (self.r.scope.read(0x1<<2) & 0x4) > 0 and not self.locked:
@@ -73,7 +88,7 @@ class DataAcquisitionService(Service):
 
                 data = self.read_data()
 
-                slow_out = self.csr.get('root_slow_value')
+                slow_out = self.csr.get('logic_slow_value')
                 slow_out = slow_out if slow_out <= 8191 else slow_out - 16384
                 data += [slow_out]
 
@@ -120,13 +135,17 @@ class DataAcquisitionService(Service):
         if hash_ == self.data_hash or self.data_hash is None:
             return False, None, None, None
         else:
-            return True, self.data_hash, self.data[:], self.data_uuid
+            return True, self.data_hash, self.data, self.data_uuid
 
     def exposed_set_ramp_speed(self, speed):
         self.ramp_speed = speed
 
     def exposed_set_lock_status(self, locked):
         self.locked = locked
+        self.confirmed_that_in_lock = False
+
+    def exposed_set_fetch_quadratures(self, fetch):
+        self.fetch_quadratures = fetch
 
     def exposed_set_csr(self, key, value):
         self.csr_queue.append((key, value))
@@ -141,7 +160,10 @@ class DataAcquisitionService(Service):
         self.data_uuid = uuid
 
     def read_data(self):
-        channel_offsets = (0x10000, 0x20000)
+        channel_offsets = [0x10000]
+        if self.fetch_quadratures:
+            channel_offsets.append(0x20000)
+
         write_pointer = self.r.scope.write_pointer_trigger
 
         def get_data(offset, addr, data_length):
@@ -152,25 +174,49 @@ class DataAcquisitionService(Service):
             else:
                 to_read_later = 0
 
-            x = self.r.scope.reads(offset+(4*addr), data_length)
-            y = x.copy()
-            y.dtype = np.int32
-            y[y>=2**13] -= 2**14
+            # raw data contains two signals:
+            #   2'h0,adc_b_rd,2'h0,adc_a_rd
+            #   i.e.: 2 zero bits, channel b (14 bit), 2 zero bits, channel a (14 bit)
+            # .copy() is required, because np.frombuffer returns a readonly array
+            raw_data = self.r.scope.reads(offset+(4*addr), data_length).copy()
+
+            # raw_data is an array of 32-bit ints
+            # we cast it to 16 bit --> each original int is split into two ints
+            raw_data.dtype = np.int16
+
+            # sign bit is at position 14, but we have 16 bit ints
+            raw_data[raw_data>=2**13] -= 2**14
+
+            # order is such that we have first the signal a then signal b
+            signals = tuple(raw_data[signal_idx::2] for signal_idx in (0, 1))
 
             if to_read_later > 0:
-                y = np.append(y, get_data(offset, 0, to_read_later))
+                additional_raw_data = get_data(offset, 0, to_read_later)
+                signals = tuple(
+                    np.append(signals[signal_idx], additional_raw_data[signal_idx])
+                    for signal_idx in range(2)
+                )
 
-            # IMPORTANT: leave this list comprehension, it is important for
-            #            performance when sending the data via rpyc
-            return [int(v) for v in y]
+            return signals
 
-        return [
-            decimate(
-                get_data(channel_offset, write_pointer, N_POINTS * self.additional_decimation),
-                self.additional_decimation
+        rv = []
+
+        for channel_offset in channel_offsets:
+            channel_data = get_data(
+                channel_offset,
+                write_pointer,
+                N_POINTS * self.additional_decimation
             )
-            for channel_offset in channel_offsets
-        ]
+
+            for sub_channel_idx in range(2):
+                rv.append(
+                    decimate(
+                        channel_data[sub_channel_idx],
+                        self.additional_decimation
+                    )
+                )
+
+        return rv
 
 
 if __name__ == '__main__':
