@@ -1,5 +1,4 @@
 import setuptools
-import subprocess
 
 import linien
 assert linien.__version__ != 'dev'
@@ -13,14 +12,14 @@ setuptools.setup(
     long_description="Have a look at the \"linien\" package for installation instructions.",
     long_description_content_type="text/x-rst",
     url="https://github.com/hermitdemschoenenleben/linien",
-    packages=['linien', 'linien.communication', 'linien.server', 'linien.server.optimization'],
+    packages=['linien', 'linien.server', 'linien.server.optimization'],
     classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
         "Operating System :: OS Independent",
     ],
     install_requires=[
-        'rpyc>=4.1.0', 'myhdl', 'click'
+        'rpyc>=4.1.0', 'myhdl', 'click', 'cma', 'plumbum'
     ],
     scripts=[
         'linien/server/linien_start_server',
